Package: BioCro
Version: 3.4.0
Date: 2026-08-20
Title: Modular Crop Growth Simulations
Description: A cross-platform representation of models as sets of equations
    that facilitates modularity in model building and allows users to harness
    modern techniques for numerical integration and data visualization.
    Documentation is provided by several vignettes included in this package;
    also see Lochocki et al. (2022) <doi:10.1093/insilicoplants/diac003>.
Authors@R: c(
    person("Justin M.", "McGrath", role = c('cre', 'aut'),
           email = "jmcgrath@illinois.edu",
           comment = c(ORCID = "0000-0002-7025-3906")),
    person("Edward B.", "Lochocki", role = 'aut',
           comment = c(ORCID = "0000-0002-4912-9783")),
    person("Yufeng", "He", role = 'aut',
           comment = c(ORCID = "0000-0001-9895-1880")),
    person("Scott W.", "Oswald", role = 'aut',
           comment = c(ORCID = "0000-0002-1906-0340")),
    person("Scott", "Rohde", role = 'aut',
           comment = c(ORCID = "0000-0001-9030-0936")),
    person("Deepak", "Jaiswal", role = 'aut',
           comment = c(ORCID = "0000-0002-4077-3919")),
    person("Megan L.", "Matthews", role = 'aut',
           comment = c(ORCID = "0000-0002-5513-9320")),
    person("Fernando E.", "Miguez", role = 'aut',
           comment = c(ORCID = "0000-0002-4627-8329")),
    person("Stephen P.", "Long", role = 'aut',
           comment = c(ORCID = "0000-0002-8501-7164")),
    person("Dan", "Wang", role = 'ctb'),
    person("David", "LeBauer", role = 'ctb',
           comment = c(ORCID = "0000-0001-7228-053X")),
    person("BioCro authors", role = "cph"),
    person("Boost Organization", role = "cph",
           comment = "Copyright holder of included Boost library")
    )
Depends: R (>= 4.1.0)
Imports: stats
Suggests: testthat (>= 3.2.0), knitr, rmarkdown, bookdown, lattice,
        deSolve, dfoptim
VignetteBuilder: knitr
SystemRequirements: C++17, GNU make
License: MIT + file LICENSE
LazyData: true
ByteCompile: TRUE
URL: https://github.com/biocro/biocro, https://biocro.org
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-08-21 13:42:17 UTC; Justin
Author: Justin M. McGrath [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-7025-3906>),
  Edward B. Lochocki [aut] (ORCID:
    <https://orcid.org/0000-0002-4912-9783>),
  Yufeng He [aut] (ORCID: <https://orcid.org/0000-0001-9895-1880>),
  Scott W. Oswald [aut] (ORCID: <https://orcid.org/0000-0002-1906-0340>),
  Scott Rohde [aut] (ORCID: <https://orcid.org/0000-0001-9030-0936>),
  Deepak Jaiswal [aut] (ORCID: <https://orcid.org/0000-0002-4077-3919>),
  Megan L. Matthews [aut] (ORCID:
    <https://orcid.org/0000-0002-5513-9320>),
  Fernando E. Miguez [aut] (ORCID:
    <https://orcid.org/0000-0002-4627-8329>),
  Stephen P. Long [aut] (ORCID: <https://orcid.org/0000-0002-8501-7164>),
  Dan Wang [ctb],
  David LeBauer [ctb] (ORCID: <https://orcid.org/0000-0001-7228-053X>),
  BioCro authors [cph],
  Boost Organization [cph] (Copyright holder of included Boost library)
Maintainer: Justin M. McGrath <jmcgrath@illinois.edu>
Repository: CRAN
Date/Publication: 2026-08-21 19:10:08 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-08-22 00:05:52 UTC; unix
Archs: BioCro.so.dSYM
